@@ -1,5 +1,5 @@
 # ytcc - The YouTube channel checker
-# Copyright (C) 2020  Wolfgang Popp
+# Copyright (C) 2021  Wolfgang Popp
 #
 # This file is part of ytcc.
 #
@@ -48,19 +48,35 @@ class CommaList(click.ParamType, Generic[T]):
     def __init__(self, validator: Callable[[str], T]):
         self.validator = validator
 
-    def convert(self, value, param, ctx) -> List[T]:
+    def convert(self, value, param, ctx) -> List[T]:  # pylint: disable=inconsistent-return-statements
         try:
             return [self.validator(elem.strip()) for elem in value.split(",")]
         except ValueError:
             self.fail(f"Unexpected value {value} in comma separated list")
 
 
+class TruncateVals(click.ParamType):
+    name = "truncate"
+
+    def convert(self, value, param, ctx) -> Union[None, str, int]:  # pylint: disable=inconsistent-return-statements
+        if value == "max":
+            return "max"
+        if value == "no":
+            return None
+        try:
+            return int(value)
+        except ValueError:
+            self.fail(f"Unexpected value {value}. Must be 'no', 'max', or an integer")
+
+
 version_text = f"""%(prog)s, version %(version)s
 
-Copyright (C) 2015-2020  {__author__}
+Copyright (C) 2015-2021  {__author__}
 This program comes with ABSOLUTELY NO WARRANTY; This is free software, and you
 are welcome to redistribute it under certain conditions.  See the GNU General
-Public Licence for details."""
+Public Licence for details.
+
+See `%(prog)s bug-report` for more detailed version information."""
 
 
 def _load_completion_conf(args: List[str]) -> None:
@@ -139,9 +155,13 @@ def tag_completion(ctx: Any, args: List[str],  # pylint: disable=unused-argument
                    " `rss` prints a RSS 2.0 feed of videos.")
 @click.option("--separator", "-s", default=",", show_default=True,
               help="Set the delimiter used in XSV format.")
+@click.option("--truncate", "-t", default="max", show_default=True, type=TruncateVals(),
+              help="Truncate the table output. 'max' truncates to terminal width, 'no' disables"
+                   " truncating, an integer N truncates to length N.")
 @click.version_option(version=__version__, prog_name="ytcc", message=version_text)
 @click.pass_context
-def cli(ctx: click.Context, conf: Path, loglevel: str, output: str, separator: str) -> None:
+def cli(ctx: click.Context, conf: Path, loglevel: str, output: str, separator: str,
+        truncate: Union[None, str, int]) -> None:
     """Ytcc - the (not only) YouTube channel checker.
 
     Ytcc "subscribes" to playlists (supported by youtube-dl) and tracks new videos published to
@@ -173,7 +193,7 @@ def cli(ctx: click.Context, conf: Path, loglevel: str, output: str, separator: s
     ctx.call_on_close(ytcc.close)
 
     if output == "table":
-        printer = TablePrinter()
+        printer = TablePrinter(truncate)
     elif output == "json":
         printer = JSONPrinter()
     elif output == "xsv":
@@ -185,15 +205,19 @@ def cli(ctx: click.Context, conf: Path, loglevel: str, output: str, separator: s
 @cli.command()
 @click.argument("name")
 @click.argument("url")
+@click.option("--reverse", is_flag=True, default=False,
+              help="Check the playlist in reverse order. This should be used for playlists where "
+                   "the latest videos are added to the end of the playlist. WARNING: Using this "
+                   "option on large playlists slows down updating!")
 @pass_ytcc
-def subscribe(ytcc: core.Ytcc, name: str, url: str):
+def subscribe(ytcc: core.Ytcc, name: str, url: str, reverse: bool):
     """Subscribe to a playlist.
 
     The NAME argument is the name used to refer to the playlist. The URL argument is the URL to a
     playlist that is supported by youtube-dl.
     """
     try:
-        ytcc.add_playlist(name, url)
+        ytcc.add_playlist(name, url, reverse)
     except BadURLException as bad_url:
         logger.error("The given URL does not point to a playlist or is not supported by "
                      "youtube-dl")
@@ -237,10 +261,28 @@ def rename(ytcc: core.Ytcc, old: str, new: str):
     try:
         ytcc.rename_playlist(old, new)
     except NameConflictError as nce:
-        logger.error("'%s'", str(nce))
+        logger.error("%s", str(nce))
         raise Exit(1) from nce
 
 
+@cli.command("reverse")
+@click.argument("playlists", nargs=-1, autocompletion=playlist_completion)
+@pass_ytcc
+def reverse_playlist(ytcc: core.Ytcc, playlists: Tuple[str, ...]):
+    """Toggle the update behavior of playlists.
+
+    Playlists updated in reverse might lead to slow updates with the `update` command.
+    """
+    for playlist in playlists:
+        try:
+            ytcc.reverse_playlist(playlist)
+        except PlaylistDoesNotExistException as err:
+            logger.error("Could not reverse playlist '%s', because it doesn't exist", playlist)
+            raise Exit(1) from err
+        else:
+            logger.info("Reversed playlist '%s'", playlist)
+
+
 @cli.command()
 @click.option("--attributes", "-a", type=CommaList(PlaylistAttr.from_str),
               help="Attributes of the playlist to be included in the output. "
@@ -248,7 +290,7 @@ def rename(ytcc: core.Ytcc, old: str, new: str):
 @pass_ytcc
 def subscriptions(ytcc: core.Ytcc, attributes: List[PlaylistAttr]):
     """List all subscriptions."""
-    if not filter:
+    if not attributes:
         printer.filter = config.ytcc.playlist_attrs
     else:
         printer.filter = attributes
@@ -282,6 +324,10 @@ def update(ytcc: core.Ytcc, max_fail: Optional[int], max_backlog: Optional[int])
     ytcc.update(max_fail, max_backlog)
 
 
+_video_attrs = click.Choice(list(VideoAttr))
+_video_attrs.name = "attribute"
+_dir = click.Choice(list(Direction))
+_dir.name = "direction"
 common_list_options = [
     click.Option(["--tags", "-c"], type=CommaList(str),
                  help="Listed videos must be tagged with one of the given tags."),
@@ -299,8 +345,10 @@ common_list_options = [
                  help="Only watched videos are listed."),
     click.Option(["--unwatched", "-u"], is_flag=True, default=False,
                  help="Only unwatched videos are listed."),
-    click.Option(["--order-by", "-o"], type=(click.Choice(VideoAttr), click.Choice(Direction)),
-                 multiple=True, help="Set the column and direction to sort listed videos.")
+    click.Option(["--order-by", "-o"], type=(_video_attrs, _dir), multiple=True,
+                 help="Set the column and direction to sort listed videos. "
+                      f"ATTRIBUTE is one of [{', '.join(VideoAttr)}]. "
+                      f"Direction is one of [{', '.join(Direction)}].")
 
 ]
 
@@ -340,7 +388,7 @@ def list_videos_impl(ytcc: core.Ytcc, tags: List[str], since: datetime, till: da
 @cli.command("list")
 @click.option("--attributes", "-a", type=CommaList(VideoAttr.from_str),
               help="Attributes of videos to be included in the output. "
-                   f"Some of [{', '.join(map(lambda x: x.value, list(VideoAttr)))}].")
+                   f"Some of [{', '.join(VideoAttr)}].")
 @pass_ytcc
 def list_videos(ytcc: core.Ytcc, tags: List[str], since: datetime, till: datetime,
                 playlists: List[str], ids: List[int], attributes: List[str],
@@ -518,14 +566,26 @@ def cleanup(ytcc: core.Ytcc, keep: Optional[int]):
 
 
 @cli.command("import")
+@click.option("--format", "-f", type=click.Choice(["opml", "csv"]), default="csv",
+              show_default=True, help="Format of the file to import.")
 @click.argument("file", nargs=1, type=click.Path(exists=True, file_okay=True, dir_okay=False))
 @pass_ytcc
-def import_(ytcc: core.Ytcc, file: Path):
-    """Import YouTube subscriptions from OPML file.
+def import_(ytcc: core.Ytcc, format: str, file: Path):  # pylint: disable=redefined-builtin
+    """Import YouTube subscriptions from an OPML or CSV file.
 
-    You can export your YouTube subscriptions at https://www.youtube.com/subscription_manager.
+    The CSV file must have three columns in following order: Channel ID, Channel URL, Channel name.
+
+    You can export your YouTube subscriptions at https://takeout.google.com. In the takeout you
+    find a CSV file with your subscriptions. To speed up the takeout export only your
+    subscriptions, not your videos, comments, etc.
+
+    The OPML export was available on YouTube some time ago and old versions of ytcc were also able
+    to export subscriptions in the OPML format.
     """
-    ytcc.import_yt_opml(file)
+    if format == "opml":
+        ytcc.import_yt_opml(file)
+    elif format == "csv":
+        ytcc.import_yt_csv(file)
 
 
 @cli.command()
@@ -564,7 +624,8 @@ def bug_report():
 
 def main():
     try:
-        cli.main(standalone_mode=False)
+        exit_code = cli.main(standalone_mode=False)
+        sys.exit(exit_code)
     except DatabaseError as db_err:
         logger.error("Cannot connect to the database or query failed unexpectedly")
         logger.debug("Unknown database error", exc_info=db_err)
