@@ -1,5 +1,5 @@
 # ytcc - The YouTube channel checker
-# Copyright (C) 2020  Wolfgang Popp
+# Copyright (C) 2021  Wolfgang Popp
 #
 # This file is part of ytcc.
 #
@@ -16,130 +16,31 @@
 # You should have received a copy of the GNU General Public License
 # along with ytcc.  If not, see <http://www.gnu.org/licenses/>.
 
+import csv
 import datetime
-import itertools
 import logging
 import os
 import sqlite3
 import subprocess
 import time
 import xml.etree.ElementTree as ET
-from collections import defaultdict
-from concurrent.futures import ThreadPoolExecutor as Pool
 from pathlib import Path
 from typing import Iterable, List, Optional, Any, Dict, Tuple, Union
 from urllib.parse import parse_qs, urlparse
 
 from ytcc import config
 from ytcc.config import Direction, VideoAttr
-from ytcc.database import Database, Video, Playlist, MappedVideo, MappedPlaylist
+from ytcc.database import Database, Video, MappedVideo, MappedPlaylist, Playlist
 from ytcc.exceptions import YtccException, BadURLException, NameConflictError, \
     PlaylistDoesNotExistException, InvalidSubscriptionFileError
-from ytcc.utils import unpack_optional, take, lazy_import
+from ytcc.updater import Updater, Fetcher, YTDL_COMMON_OPTS
+from ytcc.utils import lazy_import
 
 youtube_dl = lazy_import("youtube_dl")
 
-YTDL_COMMON_OPTS = {
-    "logger": logging.getLogger("youtube_dl")
-}
-
 logger = logging.getLogger(__name__)
 
 
-class Updater:
-    def __init__(self, db_path: str, max_backlog=20, max_fail=5):
-        self.db_path = db_path
-        self.max_items = max_backlog
-        self.max_fail = max_fail
-
-        self.ydl_opts = {
-            **YTDL_COMMON_OPTS,
-            "playliststart": 1,
-            "playlistend": max_backlog,
-            "noplaylist": False,
-            "age_limit": config.ytcc.age_limit
-        }
-
-    def get_new_entries(self, playlist: Playlist) -> List[Tuple[Any, str, Playlist]]:
-        with Database(self.db_path) as database:
-            hashes = frozenset(
-                x.extractor_hash
-                for x in database.list_videos(playlists=[playlist.name])
-            )
-
-        result = []
-        with youtube_dl.YoutubeDL(self.ydl_opts) as ydl:
-            logger.info("Checking playlist '%s'...", playlist.name)
-            try:
-                info = ydl.extract_info(playlist.url, download=False, process=False)
-            except youtube_dl.DownloadError as download_error:
-                logging.error("Failed to get playlist %s. Youtube-dl said: '%s'",
-                              playlist.name, download_error)
-            else:
-                for entry in take(self.max_items, info.get("entries", [])):
-                    e_hash = ydl._make_archive_id(entry)  # pylint: disable=protected-access
-                    if e_hash is None:
-                        logger.warning("Ignoring malformed playlist entry from %s", playlist.name)
-                    elif e_hash not in hashes:
-                        result.append((entry, e_hash, playlist))
-
-        return result
-
-    def process_entry(self, e_hash: str, entry: Any) -> Tuple[str, Optional[Video]]:
-        with Database(self.db_path) as database:
-            if database.get_extractor_fail_count(e_hash) >= self.max_fail:
-                return e_hash, None
-
-        with youtube_dl.YoutubeDL(self.ydl_opts) as ydl:
-            try:
-                processed = ydl.process_ie_result(entry, False)
-            except youtube_dl.DownloadError as download_error:
-                logging.warning("Failed to get a video. Youtube-dl said: '%s'", download_error)
-                return e_hash, None
-            else:
-                publish_date = 0.0
-                date_str = processed.get("upload_date")
-                if date_str:
-                    publish_date = datetime.datetime.strptime(date_str, "%Y%m%d").timestamp()
-
-                if processed.get("age_limit", 0) > config.ytcc.age_limit:
-                    logger.warning("Ignoring video '%s' due to age limit", processed.get("title"))
-                    return e_hash, None
-
-                logger.info("Processed video '%s'", processed.get("title"))
-
-                return e_hash, Video(
-                    url=processed["webpage_url"],
-                    title=processed["title"],
-                    description=processed.get("description", ""),
-                    publish_date=publish_date,
-                    watch_date=None,
-                    duration=processed.get("duration", -1),
-                    extractor_hash=e_hash
-                )
-
-    def update(self):
-        num_workers = unpack_optional(os.cpu_count(), lambda: 1) * 4
-
-        with Pool(num_workers) as pool, Database(self.db_path) as database:
-            playlists = database.list_playlists()
-            raw_entries = dict()
-            playlists_mapping = defaultdict(list)
-            full_content = pool.map(self.get_new_entries, playlists)
-            for entry, e_hash, playlist in itertools.chain.from_iterable(full_content):
-                raw_entries[e_hash] = entry
-                playlists_mapping[e_hash].append(playlist)
-
-            results = dict(pool.map(self.process_entry, *zip(*raw_entries.items())))
-
-            for key in raw_entries:
-                for playlist in playlists_mapping[key]:
-                    if results[key] is not None:
-                        database.add_videos([results[key]], playlist)
-                    else:
-                        database.increase_extractor_fail_count(key, max_fail=self.max_fail)
-
-
 class Ytcc:
     """The Ytcc class handles updating the RSS feeds and playing and listing/filtering videos.
 
@@ -352,11 +253,25 @@ class Ytcc:
                 logger.debug("youtube-dl failed with '%s'", ydl_err)
                 return False
 
-    def add_playlist(self, name: str, url: str) -> None:
+    @staticmethod
+    def _is_playlist_reverse(items: List[Video]) -> bool:
+        if not items or items[0].publish_date >= items[-1].publish_date:
+            return False
+
+        prev = items[0]
+        for item in items:
+            if prev.publish_date > item.publish_date:
+                return False
+            prev = item
+
+        return True
+
+    def add_playlist(self, name: str, url: str, reverse: bool = False,
+                     skip_update_check: bool = False) -> None:
         ydl_opts = {
             **YTDL_COMMON_OPTS,
             "playliststart": 1,
-            "playlistend": 2,
+            "playlistend": 10,
             "noplaylist": False,
             "extract_flat": "in_playlist"
         }
@@ -373,7 +288,7 @@ class Ytcc:
                     "URL is not supported by youtube-dl or does not exist"
                 ) from download_error
 
-            if not info.get("_type") == "playlist":
+            if info.get("_type") != "playlist":
                 logger.debug(
                     "'%s' doesn't seem point to a playlist. Extractor info is: '%s'",
                     url,
@@ -382,9 +297,6 @@ class Ytcc:
                 raise BadURLException("Not a playlist or not supported by youtube-dl")
 
             peek = list(info.get("entries"))
-            if not peek:
-                logger.warning("The playlist might be empty")
-
             for entry in peek:
                 if ydl._make_archive_id(entry) is None:  # pylint: disable=protected-access
                     raise BadURLException("The given URL is not supported by ytcc, because it "
@@ -394,8 +306,22 @@ class Ytcc:
             if not real_url:
                 raise BadURLException("The playlist URL cannot be found")
 
+            if not skip_update_check:
+                logger.info("Performing update check on 10 playlist items")
+                playlist = list(Fetcher(10).fetch(Playlist(name, real_url, reverse)))
+                if not playlist:
+                    logger.warning("The playlist might be empty")
+
+                if self._is_playlist_reverse(playlist):
+                    logger.warning(
+                        "The playlist seems to be updated in opposite order. You probably won't "
+                        "receive any updates for this playlist. Use `ytcc reverse '%s'` to change "
+                        "the update behavior of the playlist.",
+                        name
+                    )
+
         try:
-            self.database.add_playlist(name, real_url)
+            self.database.add_playlist(name, real_url, reverse)
         except sqlite3.IntegrityError as integrity_error:
             logger.debug(
                 "Cannot subscribe to playlist due to integrity constraint error: %s",
@@ -429,6 +355,14 @@ class Ytcc:
     def mark_unwatched(self, video: Union[List[int], int, MappedVideo]) -> None:
         self.database.mark_unwatched(video)
 
+    def unmark_recent(self):
+        videos = list(self.database.list_videos(
+            watched=True,
+            order_by=[(VideoAttr.WATCHED, Direction.DESC)]
+        ))
+        if videos:
+            self.mark_unwatched(videos[0])
+
     def delete_playlist(self, name: str) -> None:
         if not self.database.delete_playlist(name):
             raise PlaylistDoesNotExistException(f"Could not remove playlist {name}, because "
@@ -439,6 +373,12 @@ class Ytcc:
             raise NameConflictError("Renaming failed. Either the old name does not exist or the "
                                     "new name is already used.")
 
+    def reverse_playlist(self, playlist: str) -> None:
+        if not self.database.reverse_playlist(playlist):
+            raise PlaylistDoesNotExistException(
+                "Could not modify the playlist, because it does not exist"
+            )
+
     def list_playlists(self) -> Iterable[MappedPlaylist]:
         return self.database.list_playlists()
 
@@ -477,10 +417,44 @@ class Ytcc:
             raise InvalidSubscriptionFileError(f"{str(file)} cannot be accessed") from err
 
         root = tree.getroot()
-        for element in root.findall('.//outline[@type="rss"]'):
-            name, url = _from_xml_element(element)
+        subscriptions = (
+            _from_xml_element(element)
+            for element in root.findall('.//outline[@type="rss"]')
+        )
+        self._bulk_subscribe(subscriptions)
+
+    def import_yt_csv(self, file: Path):
+        with open(file, newline='', encoding="locale") as csvfile:
+            sample = csvfile.read(4096)
+            sniffer = csv.Sniffer()
+            dialect = sniffer.sniff(sample)
+
+            csvfile.seek(0)
+            rows = csv.reader(csvfile, dialect)
+
+            if sniffer.has_header(sample):
+                next(rows, None)
+
+            subscriptions = []
+            for row in rows:
+                # Ignore empty lines
+                if not row:
+                    continue
+
+                if len(row) != 3:
+                    raise InvalidSubscriptionFileError(
+                        f"{str(file)} has an invalid number of columns. Expecting: ID, URL, Name"
+                    )
+
+                yt_url = f"https://www.youtube.com/channel/{row[0]}/videos"
+                subscriptions.append((row[2], yt_url))
+
+            self._bulk_subscribe(subscriptions)
+
+    def _bulk_subscribe(self, subscriptions: Iterable[Tuple[str, str]]) -> None:
+        for name, url in subscriptions:
             try:
-                self.add_playlist(name, url)
+                self.add_playlist(name, url, skip_update_check=True)
             except NameConflictError:
                 logger.warning("Ignoring playlist '%s', because it already subscribed", name)
             except BadURLException:
