@@ -1,5 +1,5 @@
 # ytcc - The YouTube channel checker
-# Copyright (C) 2020  Wolfgang Popp
+# Copyright (C) 2021  Wolfgang Popp
 #
 # This file is part of ytcc.
 #
@@ -45,6 +45,7 @@ def _placeholder(elements: List[Any]) -> str:
 class Playlist:
     name: str
     url: str
+    reverse: bool
 
 
 @dataclass(frozen=True)
@@ -55,6 +56,7 @@ class Video:
     publish_date: float
     watch_date: Optional[float]
     duration: float
+    thumbnail_url: Optional[str]
     extractor_hash: str
 
     @property
@@ -74,7 +76,7 @@ class MappedPlaylist(Playlist):
 
 
 class Database:
-    VERSION = 3
+    VERSION = 5
 
     def __init__(self, path: str = ":memory:"):
         is_new_db = True
@@ -86,6 +88,8 @@ class Database:
 
         sqlite3.register_converter("integer", int)
         sqlite3.register_converter("float", float)
+        sqlite3.register_converter("boolean", lambda v: bool(int(v)))
+        sqlite3.register_adapter(bool, int)
         self.connection = sqlite3.connect(f"{path}", detect_types=sqlite3.PARSE_DECLTYPES)
         self.connection.set_trace_callback(logging_cb)
         self.connection.row_factory = sqlite3.Row
@@ -115,9 +119,10 @@ class Database:
 
             CREATE TABLE playlist
             (
-                id   INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
-                name VARCHAR UNIQUE,
-                url  VARCHAR UNIQUE
+                id      INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
+                name    VARCHAR UNIQUE,
+                url     VARCHAR UNIQUE,
+                reverse BOOLEAN
             );
 
             CREATE TABLE content
@@ -137,6 +142,7 @@ class Database:
                 duration       FLOAT,
                 publish_date   FLOAT,
                 watch_date     FLOAT,
+                thumbnail_url  VARCHAR,
                 extractor_hash VARCHAR UNIQUE
             );
 
@@ -176,10 +182,10 @@ class Database:
         self.connection.commit()
         self.connection.close()
 
-    def add_playlist(self, name: str, url: str) -> None:
-        query = "INSERT INTO playlist (name, url) VALUES (?, ?);"
+    def add_playlist(self, name: str, url: str, reverse: bool = False) -> None:
+        query = "INSERT INTO playlist (name, url, reverse) VALUES (?, ?, ?);"
         with self.connection:
-            res = self.connection.execute(query, (name, url))
+            res = self.connection.execute(query, (name, url, reverse))
             return res.rowcount > 0
 
     def delete_playlist(self, name: str) -> bool:
@@ -188,7 +194,7 @@ class Database:
             res = self.connection.execute(query, (name,))
             return res.rowcount > 0
 
-    def rename_playlist(self, oldname, newname) -> bool:
+    def rename_playlist(self, oldname: str, newname: str) -> bool:
         query = "UPDATE playlist SET name = ? WHERE name = ?"
         try:
             with self.connection:
@@ -197,18 +203,24 @@ class Database:
         except sqlite3.IntegrityError:
             return False
 
+    def reverse_playlist(self, playlist: str) -> bool:
+        query = "UPDATE playlist SET reverse = (reverse + 1) % 2 WHERE name = ?"
+        with self.connection as con:
+            res = con.execute(query, (playlist,))
+            return res.rowcount > 0
+
     def list_playlists(self) -> Iterable[MappedPlaylist]:
         query = """
-            SELECT p.id AS id, p.name AS name, p.url AS url, t.name AS tag
+            SELECT p.id AS id, p.name AS name, p.url AS url, p.reverse AS reverse, t.name AS tag
             FROM playlist AS p
                 LEFT OUTER JOIN tag AS t ON p.id = t.playlist;
             """
-        playlists: Dict[int, MappedPlaylist] = dict()
+        playlists: Dict[int, MappedPlaylist] = {}
         for row in self.connection.execute(query):
             playlist = playlists.get(row["id"])
             if playlist is None:
                 tags = [row["tag"]] if row["tag"] else []
-                playlists[row["id"]] = MappedPlaylist(row["name"], row["url"], tags)
+                playlists[row["id"]] = MappedPlaylist(row["name"], row["url"], row["reverse"], tags)
             else:
                 playlists[row["id"]].tags.append(row["tag"])
 
@@ -231,25 +243,28 @@ class Database:
     def add_videos(self, videos: Iterable[Video], playlist: Playlist) -> None:
         insert_video = """
             INSERT INTO video
-                (title, url, description, duration, publish_date, watch_date, extractor_hash)
+                (title, url, description, duration, publish_date, watch_date,
+                 thumbnail_url, extractor_hash)
             VALUES
                 (:title, :url, :description, :duration, :publish_date, :watch_date,
-                 :extractor_hash)
+                 :thumbnail_url, :extractor_hash)
             ON CONFLICT (url) DO UPDATE
                 SET title = :title,
                     url = :url,
                     description = :description,
                     duration = :duration,
                     publish_date = :publish_date,
+                    thumbnail_url = :thumbnail_url,
                     extractor_hash = :extractor_hash
             """
         if sqlite3.sqlite_version_info < (3, 24, 0):
             insert_video = """
                 INSERT OR IGNORE INTO video
-                    (title, url, description, duration, publish_date, watch_date, extractor_hash)
+                    (title, url, description, duration, publish_date, watch_date,
+                     thumbnail_url, extractor_hash)
                 VALUES
                     (:title, :url, :description, :duration, :publish_date, :watch_date,
-                     :extractor_hash)
+                     :thumbnail_url, :extractor_hash)
                 """
         insert_playlist = """
             INSERT OR IGNORE INTO content (playlist_id, video_id)
@@ -324,6 +339,7 @@ class Database:
                     VideoAttr.PUBLISH_DATE: "publish_date",
                     VideoAttr.WATCHED: "watch_date",
                     VideoAttr.DURATION: "duration",
+                    VideoAttr.THUMBNAIL_URL: "thumbnail_url",
                     VideoAttr.EXTRACTOR_HASH: "extractor_hash",
                     VideoAttr.PLAYLISTS: "playlist_name",
                 }
@@ -333,8 +349,9 @@ class Database:
                     if col is not None:
                         yield col, ord_dir
 
-            order_by_clause = "ORDER BY "
-            order_by_clause += ", ".join(f"{col} {ord_dir}" for col, ord_dir in directions())
+            order_by_clause = ", ".join(f"{col} {ord_dir}" for col, ord_dir in directions())
+            if order_by_clause:
+                order_by_clause = "ORDER BY " + order_by_clause
         return order_by_clause
 
     def list_videos(
@@ -371,9 +388,11 @@ class Database:
                    v.duration       AS duration,
                    v.publish_date   AS publish_date,
                    v.watch_date     AS watch_date,
+                   v.thumbnail_url  AS thumbnail_url,
                    v.extractor_hash AS extractor_hash,
                    p.name           AS playlist_name,
-                   p.url            AS playlist_url
+                   p.url            AS playlist_url,
+                   p.reverse        AS playlist_reverse
             FROM video AS v
                      JOIN content c ON v.id = c.video_id
                      JOIN playlist p ON p.id = c.playlist_id
@@ -393,7 +412,7 @@ class Database:
         tags = unpack_optional(tags, list)
         ids = unpack_optional(ids, list)
 
-        videos: Dict[int, MappedVideo] = dict()
+        videos: Dict[int, MappedVideo] = {}
         with self.connection as con:
             for row in con.execute(query, [since, till, *ids, *tags, *playlists]):
                 video = videos.get(row["id"])
@@ -406,12 +425,23 @@ class Database:
                         publish_date=row["publish_date"],
                         watch_date=row["watch_date"],
                         duration=row["duration"],
+                        thumbnail_url=row["thumbnail_url"],
                         extractor_hash=row["extractor_hash"],
-                        playlists=[Playlist(row["playlist_name"], row["playlist_url"])]
+                        playlists=[
+                            Playlist(
+                                row["playlist_name"],
+                                row["playlist_url"],
+                                row["playlist_reverse"]
+                            )
+                        ]
                     )
                 else:
                     videos[row["id"]].playlists.append(
-                        Playlist(row["playlist_name"], row["playlist_url"])
+                        Playlist(
+                            row["playlist_name"],
+                            row["playlist_url"],
+                            row["playlist_reverse"]
+                        )
                     )
 
         return videos.values()
